@@ -25,10 +25,13 @@
 #include <TelepathyQt/Types>
 #include <TelepathyQt/ReceivedMessage>
 
+#include <KTp/message.h>
+
 class MessagesModel : public QAbstractListModel
 {
     Q_OBJECT
     Q_ENUMS(MessageType)
+    Q_ENUMS(DeliveryStatus)
     Q_PROPERTY(bool visibleToUser READ isVisibleToUser WRITE setVisibleToUser NOTIFY visibleToUserChanged)
     Q_PROPERTY(int unreadCount READ unreadCount NOTIFY unreadCountChanged)
     Q_PROPERTY(bool shouldStartOpened READ shouldStartOpened CONSTANT)
@@ -43,7 +46,9 @@ class MessagesModel : public QAbstractListModel
         TimeRole, //QDateTime
         SenderIdRole, //string
         SenderAliasRole, //string
-        SenderAvatarRole //pixmap
+        SenderAvatarRole, //pixmap
+        DeliveryStatusRole, //MessagesModel::DeliveryStatus
+        DeliveryReportReceiveTimeRole //QDateTime
     };
 
     enum MessageType {
@@ -53,6 +58,13 @@ class MessagesModel : public QAbstractListModel
         MessageTypeNotice
     };
 
+    enum DeliveryStatus {
+        DeliveryStatusUnknown,
+        DeliveryStatusDelivered,
+        DeliveryStatusRead, // implies DeliveryStatusDelivered
+        DeliveryStatusFailed
+    };
+
     virtual QVariant data(const QModelIndex &index, int role = Qt::DisplayRole) const;
     virtual int rowCount(const QModelIndex &parent = QModelIndex()) const;
 
@@ -79,6 +91,7 @@ class MessagesModel : public QAbstractListModel
     void onMessageSent(const Tp::Message &message, Tp::MessageSendingFlags flags, const QString &messageToken);
     void onPendingMessageRemoved();
     bool verifyPendingOperation(Tp::PendingOperation *op);
+    void onHistoryFetched(const QList<KTp::Message> &messages);
 
   private:
     void setupChannelSignals(const Tp::TextChannelPtr &channel);
