@@ -18,14 +18,12 @@
     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
-
 #include "qml-plugins.h"
 
 #include <QtDeclarative/QDeclarativeItem>
 #include <QtDeclarative/QDeclarativeEngine>
 #include <QtDeclarative/QDeclarativeContext>
 
-
 #include "conversation.h"
 #include "conversations-model.h"
 #include "hide-window-component.h"
@@ -35,7 +33,10 @@
 #include "filtered-pinned-contacts-proxy-model.h"
 #include "telepathy-manager.h"
 
+#include <TelepathyQt/PendingChannelRequest>
 #include "KTp/types.h"
+#include "KTp/global-presence.h"
+#include "KTp/Models/presence-model.h"
 #include "KTp/Models/contacts-filter-model.h"
 #include "KTp/Models/contacts-model.h"
 #include "KTp/Models/accounts-list-model.h"
@@ -59,12 +60,18 @@ void QmlPlugins::registerTypes(const char *uri)
     qmlRegisterType<PinnedContactsModel>(uri, 0, 1, "PinnedContactsModel");
     qmlRegisterType<ContactPin>(uri, 0, 1, "ContactPin");
     qmlRegisterType<FilteredPinnedContactsProxyModel>(uri, 0, 1, "FilteredPinnedContactsProxyModel");
+    qmlRegisterType<KTp::GlobalPresence> (uri, 0, 1, "GlobalPresence");
+    qmlRegisterType<KTp::PresenceModel> (uri, 0, 1, "PresenceModel");
 
     qmlRegisterUncreatableType<MessagesModel> (uri, 0, 1, "MessagesModel",
         QLatin1String("It will be created once the conversation is created"));
 
     qmlRegisterType<TelepathyManager>();
     qmlRegisterType<ConversationsModel>();
+
+    qmlRegisterType<Tp::PendingChannelRequest>();
+    qRegisterMetaType<Tp::Presence>();
+    qRegisterMetaType<KTp::Presence>();
     qRegisterMetaType<Tp::AccountManagerPtr>();
     qRegisterMetaType<KTp::ContactPtr>();
     qRegisterMetaType<Tp::AccountPtr>();
