@@ -43,6 +43,15 @@ KTp::Contact::Contact(Tp::ContactManager *manager, const Tp::ReferencedHandles &
     connect(manager->connection().data(), SIGNAL(invalidated(Tp::DBusProxy*,QString,QString)), SIGNAL(invalidated()));
     connect(this, SIGNAL(avatarTokenChanged(QString)), SLOT(invalidateAvatarCache()));
     connect(this, SIGNAL(avatarDataChanged(Tp::AvatarData)), SLOT(invalidateAvatarCache()));
+    connect(this, SIGNAL(presenceChanged(Tp::Presence)), SLOT(onPresenceChanged(Tp::Presence)));
+}
+
+void KTp::Contact::onPresenceChanged(const Tp::Presence &presence)
+{
+    Q_UNUSED(presence)
+    /* Temporary workaround for upstream bug https://bugs.freedesktop.org/show_bug.cgi?id=55883)
+     * Close https://bugs.kde.org/show_bug.cgi?id=308217 when fixed upstream */
+    Q_EMIT clientTypesChanged(clientTypes());
 }
 
 QString KTp::Contact::accountUniqueIdentifier() const
